"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var events_1 = require('events');
var graphql_1 = require('graphql');
var validation_1 = require('./validation');
var FilteredPubSub = (function () {
    function FilteredPubSub() {
        this.ee = new events_1.EventEmitter();
        this.subscriptions = {};
        this.subIdCounter = 0;
    }
    FilteredPubSub.prototype.publish = function (triggerName, payload) {
        this.ee.emit(triggerName, payload);
    };
    FilteredPubSub.prototype.subscribe = function (triggerName, filterFunc, handler) {
        var onMessage = function (data) { return filterFunc(data) ? handler(data) : null; };
        this.ee.addListener(triggerName, onMessage);
        this.subIdCounter = this.subIdCounter + 1;
        this.subscriptions[this.subIdCounter] = [triggerName, onMessage];
        return this.subIdCounter;
    };
    FilteredPubSub.prototype.unsubscribe = function (subId) {
        var _a = this.subscriptions[subId], triggerName = _a[0], onMessage = _a[1];
        delete this.subscriptions[subId];
        this.ee.removeListener(triggerName, onMessage);
    };
    return FilteredPubSub;
}());
exports.FilteredPubSub = FilteredPubSub;
var ValidationError = (function (_super) {
    __extends(ValidationError, _super);
    function ValidationError(errors) {
        _super.call(this);
        this.errors = errors;
        this.message = 'Subscription query has validation errors';
    }
    return ValidationError;
}(Error));
exports.ValidationError = ValidationError;
;
var SubscriptionManager = (function () {
    function SubscriptionManager(options) {
        this.pubsub = new FilteredPubSub();
        this.schema = options.schema;
    }
    SubscriptionManager.prototype.publish = function (triggerName, payload) {
        this.pubsub.publish(triggerName, payload);
    };
    SubscriptionManager.prototype.subscribe = function (options) {
        var _this = this;
        if (!options.operationName) {
            throw new Error('Must provide operationName');
        }
        var parsedQuery = graphql_1.parse(options.query);
        var errors = graphql_1.validate(this.schema, parsedQuery, graphql_1.specifiedRules.concat([validation_1.subscriptionHasSingleRootField]));
        if (errors.length) {
            throw new ValidationError(errors);
        }
        var args = options.variables;
        var triggerName = options.operationName;
        var filterFunc = function () { return true; };
        var onMessage = function (rootValue) {
            try {
                graphql_1.execute(_this.schema, parsedQuery, rootValue, context, options.variables, options.operationName).then(function (data) { return options.callback(null, data); });
            }
            catch (e) {
                options.callback(e);
            }
        };
        return this.pubsub.subscribe(triggerName, filterFunc, onMessage);
    };
    SubscriptionManager.prototype.unsubscribe = function (subId) {
        this.pubsub.unsubscribe(subId);
    };
    return SubscriptionManager;
}());
exports.SubscriptionManager = SubscriptionManager;
//# sourceMappingURL=pubsub.js.map